// ==================================================================================
//
// RTOS_Motor_Test
//
// Stud release
// Need to set stack to 4096 in config
// Simple test of each motor.
//
// CBP 12-12-17
//
// ==================================================================================

#include <Arduino_FreeRTOS.h>
#include <timers.h>
#include <Servo.h>

#define DEBUG false

Servo servoLeft;
Servo servoRight;
volatile int driveL,driveR;

TickType_t ticksPerSec;
    
void setup() {  

    Serial.begin(115200);
    Serial.println(F("In Setup function"));
    ticksPerSec = pdMS_TO_TICKS(1000);
    Serial.print("Ticks per sec = ");  
    Serial.println(ticksPerSec);  

    // Connections to the motors
    servoLeft.attach(12);
    servoRight.attach(13);    

    // First task specifies a sequence of motor drives
    xTaskCreate(MotorTest, "MotorTest", 300, NULL, 2, NULL);  
  
    // Second task actually drives motors
    xTaskCreate(MotorDrive, "MotorDrive", 300, NULL, 2, NULL);  
     
    // Create a task to monitor motor drive
    xTaskCreate(Monitor, "Monitor", 300, NULL, 2, NULL); 

}

// This task sends current vals of driveL and driveR to the motors
static void Monitor(void* pvParameters) {
  while(1) {
    cprintf("Monitor Task: driveL = %d, driveR = %d\n",driveL,driveR);
    vTaskDelay(30);     
  }
}


void loop() {

}

// This task sends current vals of driveL and driveR to the motors
static void MotorDrive(void* pvParameters) {
  while(1) {
    servoLeft.writeMicroseconds(1500 + driveL);
    servoRight.writeMicroseconds(1500 - driveR);
    vTaskDelay(30);       
  } 
}

// This task contains a sequence of drives, picked up by the
// MotorDrive task.

static void MotorTest(void* pvParameters) {
  while(1) {
    driveL = 40;
    driveR = 40;
    vTaskDelay(4000/portTICK_PERIOD_MS);
    driveL = -40;
    driveR = -40;
    vTaskDelay(4000/portTICK_PERIOD_MS);     
    driveL = 40;
    driveR = -40;
    vTaskDelay(4000/portTICK_PERIOD_MS);     
    driveL = -40;
    driveR =  40;
    vTaskDelay(4000/portTICK_PERIOD_MS);     
    
  } 
}
