// ======================================================
//
// RTOS_Release_7
// CBP 11-03-22
//
// Another mutex example this time protecting a shared
// variable. The value of sum should be zero.
// Task1 increments sum Task2 decrements sum and Task3
// prints out sum.
//
// ======================================================

#include <Arduino_FreeRTOS.h>
#include "semphr.h"
TickType_t ticksPerSec;
float msPerTick;
TaskHandle_t handle1;
TaskHandle_t handle2;
TaskHandle_t handle3;
unsigned long sum = 0;
volatile boolean done1 = false;
volatile boolean done2 = false;

SemaphoreHandle_t xMutex;
  
void setup() {
  Serial.begin(115200);
  Serial.println(F("In Setup function"));
  ticksPerSec = 1000/portTICK_PERIOD_MS;
  Serial.print("Ticks per sec = ");
  Serial.println(ticksPerSec); 
  msPerTick = 1000/ticksPerSec;
  Serial.print("Millisec/tick = ");
  Serial.println(msPerTick);    
  pinMode(4,OUTPUT);
  pinMode(5,OUTPUT);
  // Create two tasks 
  xTaskCreate(MyTask1, "Task1",300, NULL, 1, &handle1);
  xTaskCreate(MyTask2, "Task2", 300, NULL, 1, &handle2); 
  xTaskCreate(MyTask3, "Task3", 300, NULL, 1, &handle3);
  xMutex = xSemaphoreCreateMutex();
  sum = 0;
}

void loop(){
  // DO nothing
}

static void MyTask1(void* pvParameters){
  while(1) { // --------------------------------------
    Serial.println("Task1");
    
    for(long i=0;i<100000;i++) {
      sum = sum + 1;
    }
    done1 = true;
   
    vTaskDelay(30);         
    vTaskSuspend(handle1);

  }  // ----------------------------------------------
}

static void MyTask2(void* pvParameters){
  while(1) { // --------------------------------------
    Serial.println("Task2");
    
    for(long i=0;i<100000;i++) {
      sum = sum - 1;
    }

    done2 = true;

    vTaskDelay(30); 
    vTaskSuspend(handle2);  

  }  // ----------------------------------------------
}

static void MyTask3(void* pvParameters){
  while(1) { // --------------------------------------
    if( done1 && done2) {
      Serial.print("Task3 Sum = "); Serial.println(sum);
       xSemaphoreTake(xMutex,portMAX_DELAY);
      sum = 0;
      done1 = false;
      done2 = false;      
      xSemaphoreGive(xMutex);
      vTaskDelay(1000/portTICK_PERIOD_MS);   
      vTaskResume(handle1);
      vTaskResume(handle2); 
    }

    vTaskDelay(30); 

  }  // ----------------------------------------------
}
