// =============================================================================
//
// RTOS_USonic_Rel
// Student Release
// HC_SRO4 Pulse generator using Hardware Timer2 and RTOS Timer
// CBP 10-03-22
//
// =============================================================================

#include <Arduino_FreeRTOS.h>
#include <timers.h>

#define HC_SR04_SET_AND_FIRE(cycles) {uint8_t m=0xff-(cycles-1); OCR2B=m;TCNT2 =m-1;}
#define HC_SR04_SET_WIDTH(cycles) (OCR2B = 0xff-(cycles-1))

#define RxPin 2
#define TxPin 9

volatile unsigned long startTime;
volatile unsigned long endTime;
volatile boolean finished = true;
volatile float dist = 1000;

BaseType_t xTimerStarted;
TimerHandle_t xTimer;

void setup() {  
    Serial.begin(115200);
    Serial.println(F("In Setup function"));
    
    // Ultrasonic Sensor pins
    pinMode(TxPin,OUTPUT);
    pinMode(RxPin,INPUT);
    
    xTaskCreate(Monitor,"Monitor", 300,NULL,2,NULL);
    
    // Create interrupt to receive rising and falling edges of US pulse
    attachInterrupt(digitalPinToInterrupt(2), ExternalInterrupt, CHANGE); 
        
    // Create a timer to drive HC-SR04 US pinger
    xTimer = xTimerCreate("AutoReload",1000/portTICK_PERIOD_MS,pdTRUE,NULL,timerCallBack);    
    pulseSetup(1);
    xTimerStarted = xTimerStart(xTimer,0);    
}

static void Monitor(void* pvParameters) {
  while(1){
    cprintf("Monitor Task: dist = %f\n",dist);
    vTaskDelay(100/portTICK_PERIOD_MS); 
  }
  
}

static void timerCallBack(TimerHandle_t xTimer) {
  // Send out a pulse only if we've finished processing the previous
    if(finished) {         
      HC_SR04_SET_AND_FIRE(159);  // 159 x 63 nS = 10 muS     
    } else {    
    }  

}
void loop() {

}

 // Receives US pulse change on pin 2
 void ExternalInterrupt(){
  int inVal = digitalRead(RxPin);
  switch(inVal) {
    case HIGH:              // The pulse goes high
      startTime = micros();
      break;
    case LOW:               // The pulse goes low
      endTime = micros();  
      dist = (float)(endTime - startTime)*330.0/20000.0;  
      finished = true; 
      // USonic hardware needs this 
      delay(60);   
      break;
  }

}

// Program hardware timer 2 to emit a 10 mu-second pulse needed by US sensor
void pulseSetup(uint8_t cycles) {
  finished = false;
  TCCR2B =  0;     
  TCNT2 = 0x00;    
  OCR2A = 0;     
  HC_SR04_SET_WIDTH(cycles);   
  TCCR2A = _BV(COM2B0) | _BV(COM2B1) | _BV(WGM20) | _BV(WGM21); 
  TCCR2B = _BV(WGM22)| _BV(CS20);       
  
}
